/*
 * Created on 06.12.2003
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package appfacade;
import domain.*;
import java.util.*;

/**
 * @author matthias
 *
 * Create a separate object of PatientDataFacade for every dialog dealing with 
 * patient data. Otherwise you'll see the same patient's data in all your dialogs 
 * as PatientDataFacade is only able to hold one patient's data at the same time.
 */

public class PatientDataFacade {
	private Patient _subject;
	private Date _date;
	private Hashtable _attributes = new Hashtable();

	public PatientDataFacade() {
		setupAttributes();
	};

	static public void setupObservationToolkit() {
		setupGenderToolkit();
		setupBreathingRateToolkit();
		setupHeartRateToolkit();
		setupTemperatureToolkit();
		setupLevelOfConsciousnessToolkit();
		setupBloodGroupToolkit();
	};

	static private void setupGenderToolkit() {
		PhenomenonType sex = new PhenomenonType("gender").persist();
		String[] sexes = { "male", "female" };
		sex.setPhenomena(sexes);
	};

	static private void setupBreathingRateToolkit() {
		Unit cpm = new Unit("cycles/min").persist();
		PhenomenonType breathingRate =
			new PhenomenonType("breathing rate").persist();

		new Phenomenon("slow", breathingRate).range(
			new QuantityRange(null, false, new Quantity(7, cpm), false));
		new Phenomenon("normal", breathingRate).range(
			new QuantityRange(
				new Quantity(7, cpm),
				true,
				new Quantity(12, cpm),
				true));
		new Phenomenon("fast", breathingRate).range(
			new QuantityRange(new Quantity(12, cpm), false, null, false));
	};

	static private void setupHeartRateToolkit() {
		Unit bpm = new Unit("beats/min").persist();
		PhenomenonType heartRate = new PhenomenonType("heart rate").persist();

		new Phenomenon("slow", heartRate).range(
			new QuantityRange(null, false, new Quantity(60, bpm), false));
		new Phenomenon("normal", heartRate).range(
			new QuantityRange(
				new Quantity(60, bpm),
				true,
				new Quantity(80, bpm),
				true));
		new Phenomenon("fast", heartRate).range(
			new QuantityRange(new Quantity(80, bpm), false, null, false));
	};

	static private void setupTemperatureToolkit() {
		Unit degree = new Unit("degree Celsius").persist();
		PhenomenonType temperature =
			new PhenomenonType("temperature").persist();

		new Phenomenon("low", temperature).range(
			new QuantityRange(null, false, new Quantity(36.5, degree), false));
		new Phenomenon("normal", temperature).range(
			new QuantityRange(
				new Quantity(36.5, degree),
				true,
				new Quantity(37.5, degree),
				true));
		new Phenomenon("high", temperature).range(
			new QuantityRange(new Quantity(37.5, degree), false, null, false));
	};

	static private void setupLevelOfConsciousnessToolkit() {
		PhenomenonType loc =
			new PhenomenonType("level of consciousness").persist();
		new Phenomenon("alert", loc);
		new Phenomenon("verbal", loc);
		new Phenomenon("pain", loc);
		new Phenomenon("unresponsive", loc);
	};

	static private void setupBloodGroupToolkit() {
		PhenomenonType bloodGroup = new PhenomenonType("blood group").persist();
		new Phenomenon("0", bloodGroup);
		new Phenomenon("A", bloodGroup);
		new Phenomenon("B", bloodGroup);
		new Phenomenon("AB", bloodGroup);
	};

	private void setupAttributes() {
		_attributes.put(
			"breathing rate",
			new MeasurementAttribute(
				this,
				PhenomenonType.get("breathing rate"),
				Unit.get("cycles/min")));
		_attributes.put(
			"heart rate",
			new MeasurementAttribute(
				this,
				PhenomenonType.get("heart rate"),
				Unit.get("beats/min")));
		_attributes.put(
			"temperature",
			new MeasurementAttribute(
				this,
				PhenomenonType.get("temperature"),
				Unit.get("degree Celsius")));
		_attributes.put(
			"level of consciousness",
			new CategoryAttribute(
				this,
				PhenomenonType.get("level of consciousness")));
		_attributes.put(
			"blood group",
			new CategoryAttribute(this, PhenomenonType.get("blood group")));
	};

	public void load() {
		_date = null;
		loadAttributes();
	};

	private void loadAttributes() {
		Enumeration e = _attributes.elements();
		while (e.hasMoreElements()) {
			FacadeAttribute each = (FacadeAttribute) e.nextElement();
			each.load();
		};
	};

	public void save() {
		if (_date == null)
			_date = GregorianCalendar.getInstance().getTime();
		saveAttributes();
	};

	private void saveAttributes() {
		Enumeration e = _attributes.elements();
		while (e.hasMoreElements()) {
			FacadeAttribute each = (FacadeAttribute) e.nextElement();
			each.save();
		};
	};

	public String name() {
		return _subject.name();
	};
	public String gender() {
		return _subject.phenomenonOf("gender").name();
	};
	public Date date() {
		return _date;
	};
	public void date(Date date) {
		_date = date;
	};

	// subject handling
	public Enumeration getLegalSubjects() {
		// returns an enumeration of strings
		return Patient.getAll();
	};
	
	/*
	 * Set the subject of the facade. The subject is an instance of Patient. If the instance 
	 * required by subjectname does not exist, it is created. This way it is possible 
	 * to create new patients in a way transient to the GUI programmer. Because, in any 
	 * case he calls this function, no need to check himself if the subjectname determines 
	 * an already existing instance of Patient.
	 */
	public void subject(String subjectname) {
		if (Patient.get(subjectname) == null) //Patient must be created first
			new Patient(subjectname).persist();
		subject(Patient.get(subjectname));
	};

	public void subject(Patient subject) {
		_subject = subject;
		loadAttributes();
	};
	
	public Patient subject() {
		return _subject;
	};

	// gender handling

	// breathing rate handling
	public String breathingRate() {
		return breathingRateAttribute().value();
	};
	public void breathingRate(String newRate) {
		breathingRateAttribute().value(newRate);
	};
	public String breathingRateName() {
		return breathingRateAttribute().name();
	};
	private MeasurementAttribute breathingRateAttribute() {
		return (MeasurementAttribute) _attributes.get("breathing rate");
	};

	// heart rate handling
	public String heartRate() {
		return heartRateAttribute().value();
	};
	public void heartRate(String newRate) {
		heartRateAttribute().value(newRate);
	};
	public String heartRateName() {
		return heartRateAttribute().name();
	};
	private MeasurementAttribute heartRateAttribute() {
		return (MeasurementAttribute) _attributes.get("heart rate");
	};

	// temperature handling
	public String temperature() {
		return temperatureAttribute().value();
	};
	public void temperature(String newTemperature) {
		temperatureAttribute().value(newTemperature);
	};
	public String temperatureName() {
		return temperatureAttribute().name();
	};
	private MeasurementAttribute temperatureAttribute() {
		return (MeasurementAttribute) _attributes.get("temperature");
	};

	// level of consciousness handling
	public String levelOfConsciousness() {
		return levelOfConsciousnessAttribute().value();
	};
	public void levelOfConsciousness(Object newValue) {
		if (newValue instanceof String)
			levelOfConsciousnessAttribute().value((String) newValue);
	};
	public Enumeration levelOfConsciousnessValues() {
		return levelOfConsciousnessAttribute().legalValues();
	};
	private CategoryAttribute levelOfConsciousnessAttribute() {
		return (CategoryAttribute) _attributes.get("level of consciousness");
	};

	// blood group handling
	public String bloodGroup() {
		return bloodGroupAttribute().value();
	};
	public void bloodGroup(String newBloodGroup) {
		bloodGroupAttribute().value(newBloodGroup);
	};
	public Enumeration bloodGroupValues() {
		return bloodGroupAttribute().legalValues();
	};
	private CategoryAttribute bloodGroupAttribute() {
		return (CategoryAttribute) _attributes.get("blood group");
	};
};